#include <errno.h>
#include <fcntl.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <mosquitto.h>

// Server connection parameters
#define MQTT_HOSTNAME "localhost"
#define MQTT_PORT 1883
#define MQTT_USERNAME "admin"
#define MQTT_PASSWORD "admin"
#define MQTT_TOPIC "test"

/*
 * Start here
 */
int main (int argc, char **argv)
{
	struct mosquitto *mosq = NULL; // Create mqtt client
 
	float metric1, metric2; // Default of two sensor-readings can add more depending on type of sensor
	mosquitto_lib_init(); 
	mosq = mosquitto_new (NULL, true, NULL); // Connect to mosquitto
	if (!mosq)
	{
		fprintf (stderr, "Can't initialize Mosquitto library\n");
		exit (-1);
	}

//	mosquitto_username_pw_set (mosq, MQTT_USERNAME, MQTT_PASSWORD);

	int ret = mosquitto_connect (mosq, MQTT_HOSTNAME, MQTT_PORT, 0);
	if (ret)
	{
		fprintf (stderr, "Can't connect to Mosquitto server\n");
		exit (-1);
	}

	int id; // Id number
	char type[6]; // Sensor type
	char idString[13]; // Id String to uniquely identify the sensor
	char text[84]; // Record message
	float bound = 101; // Maximum data-range, can adjust readings go over 101
	time_t timer; // Used to create random numbers

	// The record message-format will depend on the type of record that the user wants to be displayed by default 
	// there are three: "light, watts, and volts"

	if(argc > 1) {
		while(1)
		{
			srand((unsigned) time(&timer)); // Initiate random number generator
			metric1=((float) rand()/ (float)(RAND_MAX) * 101.0); // Get two random numbers
			metric2=((float) rand()/ (float)(RAND_MAX) * 101.0);
			id = (rand() % 10) + 1; // Get a random id number

			if(strcmp(argv[1],"light") == 0){
				sprintf(type,"light"); // Set type
				sprintf(idString, "A_%d",id); // Set id-string
				sprintf (text, "{ id: %s, type: %s, light: %f, sound : %f }", idString,type, metric1,metric2); 
				// Create message as a JSON-string
			} 

			else if(strcmp(argv[1],"volts") == 0) {
				sprintf(type,"volts"); // Set type
				sprintf(idString, "B_%d",id); // Set id-string
				sprintf(text, "{ id: %s, type: %s , volts : %f, amps : %f }",idString, type,metric1, metric2);
				// Create message as a JSON-string
			} 

			else if(strcmp(argv[1],"watts") == 0) {
				sprintf(type,"watts"); // Set type
				sprintf(idString, "C_%d",id); // Set id-string
				sprintf(text, "{ id: %s, type: %s, watts : %f, heat : %f }",idString, type, metric1, metric2);
				// Create message as a JSON-string
			} 

			// Publish the message to the topic
			ret = mosquitto_publish (mosq, NULL, MQTT_TOPIC,
					strlen (text), text, 0, false);
			printf("%s\n", text); // Display message
			if (ret)
			{
				fprintf (stderr, "Can't publish to Mosquitto server\n");
				exit (-1);
			}
			sleep(1);
		}
	} else 
		printf("You must add a sensor type input when running this client program\n");
	// We need a short delay here, to prevent the Mosquitto library being
	//  torn down by the operating system before all the network operations
	//  are finished.
	sleep (1);

	// Tidy up
	mosquitto_disconnect (mosq);
	mosquitto_destroy (mosq);
	mosquitto_lib_cleanup();

	return 0;
}
